#!/bin/env ruby
# encoding: utf-8

require_relative 'row'
require_relative 'box_drawing_chars'
require_relative 'soperf_helpers'
require_relative 'color_helper'
require_relative 'render_job'
require_relative 'chart_container'

module SoPerf
  class Chart
    attr_accessor :data_hash, :duration, :scale
    include BoxDrawingChars
    include TerminalHelper
    include TimeHelper
    include ValueFinder

    def initialize(data_hash)
      @data_hash = Hash[data_hash.sort_by do |_, arr| #sort the data by start time of first range
                          arr = [arr] unless arr.is_a?(Array)
                          arr.map { |range| range.first }.max
                        end]
      @chart     = {}
      set_up_rows
      @duration     = data_range(:round) #Find the full range of the data, rounded up
      @longest_name = data_hash.map { |k, _| k.to_s.length }.max #The longest name of the results
      @spacer       = Spacer.new(@longest_name)
      @text_info    = @data_hash.map { |k, v| "#{@chart[k].name(true)}: #{translate_time(v.first)}-#{translate_time(v.last)}" }
      @forced_width = nil
    end

    def ideal_scale
      (usable_space.to_f)/duration
    end

    def data_range(round=:round)
      range = data_hash.map { |_, v| v.last }.max
      round==:round ? ((range/10.0).ceil*10) : range
    end

    def set_up_rows
      ColorHelper.reset_color_index
      data_hash.each { |k, v| @chart[k] = Row.new(k, v) }
    end

    def render(options_hash={})
      options_hash[:color] = true unless options_hash.has_key?(:color)
      @forced_width        = options_hash[:width] if options_hash.has_key?(:width)
      show_summary         = options_hash.has_key?(:summary) ? options_hash[:summary] : true
      @names               = @chart.map { |_, v| v.name(true) }
      options_hash.merge!({duration: data_range, longest_name: @longest_name,
                           spacer:   @spacer, names: @names})

      @render_job = RenderJob.new(options_hash)

      result = ''
      set_up_rows
      @chart.each_pair { |_, row| result << row.render(@render_job) + NEWLINE }
      contained = ChartContainer.new(@render_job).container(result)
      rendered  = "#{contained}\n#{(show_summary ? print_text_info(@render_job) : '')}"
      reset
      options_hash[:color] ? rendered : (rendered.uncolorize)
    end

    def scaled_tick_locations
      @scaled_tick_locations ||= tick_locations.map { |val| val*scale }
    end

    def print_text_info(render_job)
      # return ''
      result = @spacer.space(-1)

      @text_info.length.times do |i|
        if (result.split("\n")[-1] + @text_info[i]).uncolorize.length > render_job.width
          result << "\n" + @spacer.space(-1) + @text_info[i] + ", "
        else
          result << "#{@text_info[i]}, "
        end
      end
      result[0..-3]
    end

    def reset
      @scaled_tick_locations = nil
      @tick_locations        = nil
      @tick_freq             = nil
      @pretty_tick_values    = nil
      @forced_width          = nil
      @show_summary          = nil
    end

  end
end
