require_relative 'soperf_helpers'

module SoPerf
  class RenderJob
    include TerminalHelper
    include ValueFinder
    attr_accessor :width, :scale, :raw_duration, :duration, :spacer,
                  :tick_frequency, :pretty_tick_values, :tick_locations,
                  :each_space_values, :scaled_tick_locations, :use_color,
                  :names, :longest_name

    def initialize(options_hash)
      options = default_render_options.merge(options_hash)
      raise StandardError if options[:duration].nil?

      @names                 = options[:names]
      @spacer                = options[:spacer]
      @use_color             = options[:color]
      @width                 = options.has_key?(:width) ? options[:width] : usable_space(terminal_width, options[:longest_name])
      puts terminal_width
      @raw_duration          = options[:duration]
      @duration              = ((options[:duration]/10.0).ceil*10)
      @scale                 = @width/@duration.to_f
      @tick_frequency        = tick_frequency
      @pretty_tick_values    = pretty_tick_values
      @tick_locations        = tick_locations
      @each_space_values     = values_for_each_space
      @scaled_tick_locations = scaled_tick_locations
      @longest_name          = options[:longest_name]
    end

    def default_render_options
      { duration:     nil,
        longest_name: 0,
        color:        true,
        summary:      true,
        spacer:       Spacer.new }
    end

    def values_for_each_space
      result = []
      count  = 0
      (@width+1).times do
        result << count
        count += @duration/@width.to_f
      end
      @pretty_tick_values.each do |val|
        index         = closest(result, val)
        result[index] = val
      end
      result
    end

    def tick_locations
      result      = [] #locations of the ticks
      tick_values = [] #value for every single space on the chart
      (@width+1).times { |i| tick_values<<((@duration/@width.to_f)*i) }
      @pretty_tick_values.each { |val| result<<closest(tick_values, val) }
      result.map { |elem| elem/scale }
    end

    def tick_frequency
      result = 10
      while (result*10)/duration<@raw_duration/5 && result*10 < @raw_duration
        result*=10
      end
      result/=2 if @scale > 1 #If we've upscaled at all, add more tick lines to compensate.
      result/=2 if @duration/result < @width/20 #If we've got too few lines, add more!
      result
    end

    def pretty_tick_values
      arr = [0]
      (@duration/@tick_frequency).times { arr<<arr.last+@tick_frequency }
      arr.map { |val| val*@scale }
      arr
    end

    def usable_space(base_width, longest_name) #Space for the chart,
      base_width - longest_name - 10 #accounting for printing row names
    end

    def scaled_tick_locations
      @tick_locations.map { |val| val*@scale }
    end
  end
end
